// Imports
import * as Auth from "/services/auth/auth.js";
import * as results from "/services/election/calculate-election-generic.js";
import * as PartyModel from "/services/model/party-model.js";
import * as CandidateListModel from "/services/model/candidateList-model.js";
import * as AreaModel from "/services/model/area-model.js";
import * as ResultsZone from "/components/visualization/results-zone.js";
import * as ResultsGeneral from "/components/visualization/results-general.js";

export async function mount(where, round) {
  const resultComponent = new ResultComponent(round);
  await resultComponent.mount(where);
}

class ResultComponent {
  constructor(round) {
    this.round = round;
    this.PartyModel = PartyModel.getPartyModel();
    this.CandidateListModel = CandidateListModel.getCandidateListModel();
    this.AreaModel = AreaModel.getAreaModel();
  }

  async mount(where) {
    this.PartyModel.current_user = await Auth.GetUser();
    this.CandidateListModel.current_user = await Auth.GetUser();
    this.AreaModel.current_user = await Auth.GetUser();
    const mountpoint = where;
    document.getElementById(mountpoint).innerHTML = /* HTML */ `
      <div class="tabs is-boxed is-toggle is-fullwidth">
        <ul>
          <li id="general" class="is-active">
            <a>
              <span>Général</span>
            </a>
          </li>
          <li id="areas">
            <a>
              <span>Circonscriptions</span>
            </a>
          </li>
          <li id="sections">
            <a>
              <span>Villes</span>
            </a>
          </li>
        </ul>
      </div>
      <div class="control filter">
        <label class="radio">
          <input type="radio" name="filter" value="partial" checked />
          Partiel
        </label>
        <label class="radio">
          <input type="radio" name="filter" value="completed" />
          Complété
        </label>
        <label class="radio">
          <input type="radio" name="filter" value="validated" />
          Validé
        </label>
      </div>
      <div id="results-general" class="columns"></div>
      <div id="results-zone" class="columns" style="display: none;"></div>
    `;
    this.calculator = await results.mountCalculator(this.round);
    await this.calculateResults();
    this.resultsZone = await ResultsZone.mount("results-zone", this);
    this.resultsGeneral = await ResultsGeneral.mount("results-general", this);
    this.handleDom();
  }

  progressBarTemplate(candidateList, color, electedsNumber) {
    return /* HTML */ `<div class="progressBar">
      <div
        class="progressBarValue"
        style="background-color : ${color}; width : ${candidateList.Percentage}%"
      >
        ${candidateList.Name +
        " (" +
        (candidateList.VoiceNumber != null
          ? candidateList.VoiceNumber + " votes soit : "
          : "") +
        candidateList.Percentage +
        "%" +
        (electedsNumber != null ? ", " + electedsNumber + " élus" : "")})
      </div>
    </div>`;
  }

  handleDom() {
    let resultHandler = this;
    document
      .getElementById("general")
      .addEventListener("click", async function () {
        document.getElementById("results-zone").style.display = "none";
        document.getElementById("results-general").style.display = "flex";
        await resultHandler.calculateResults();
        document.getElementById("sections").setAttribute("class", "");
        document.getElementById("areas").setAttribute("class", "");
        document.getElementById("general").setAttribute("class", "is-active");
      });
    document
      .getElementById("areas")
      .addEventListener("click", async function () {
        document.getElementById("results-zone").style.display = "flex";
        document.getElementById("results-general").style.display = "none";
        resultHandler.zone = "areas";
        await resultHandler.calculateResults();
        resultHandler.resultsZone.displayResults();
        document.getElementById("sections").setAttribute("class", "");
        document.getElementById("general").setAttribute("class", "");
        document.getElementById("areas").setAttribute("class", "is-active");
      });
    document
      .getElementById("sections")
      .addEventListener("click", async function () {
        document.getElementById("results-zone").style.display = "flex";
        document.getElementById("results-general").style.display = "none";
        resultHandler.zone = "sections";
        await resultHandler.calculateResults();
        resultHandler.resultsZone.displayResults();
        document.getElementById("areas").setAttribute("class", "");
        document.getElementById("general").setAttribute("class", "");
        document.getElementById("sections").setAttribute("class", "is-active");
      });

    let radioButtons = document.getElementsByName("filter");
    for (var i = 0; i < radioButtons.length; i++) {
      radioButtons[i].addEventListener("click", async (e) => {
        await this.calculateResults();
        this.resultsZone.displayResults();
        this.resultsGeneral.displayRoundResults();
      });
    }

    document.addEventListener(
      "input",
      function __listener(event) {
        if (event.target.id == "select-areas") {
          if (event.target.value != 0) {
            let area = resultHandler.results.areasResults.find(
              (area) => area.ID == event.target.value
            );

            if (resultHandler.zone === "areas")
              resultHandler.resultsZone.displayZoneResults(area);
            else if (resultHandler.zone === "sections") {
              resultHandler.refreshSections(area);
              resultHandler.resultsZone.displayZoneResults(area.Sections[0]);
            }
          }
        } else if (event.target.id == "select-sections") {
          let area = resultHandler.results.areasResults.find(
            (area) => area.ID == document.getElementById("select-areas").value
          );
          let section = area.Sections.find(
            (section) => section.ID == event.target.value
          );
          resultHandler.resultsZone.displayZoneResults(section);
        }
      },
      false
    );

    this.interval = setInterval(async () => {
      this.calculateResults();
      this.resultsZone.displayResults();
      this.resultsGeneral.displayRoundResults();
    }, 10000);
  }

  async calculateResults() {
    this.filter = document.querySelector('input[name="filter"]:checked').value;
    this.results = await this.calculator.calculateResults(this.filter);
  }

  displayRoundResults() {
    document.getElementById(
      "detailed-results"
    ).innerHTML = `<h5 class="title is-5">Résultats détaillés du tour</h5>`;
    for (let i in this.results.roundResults) {
      document.getElementById(
        "detailed-results"
      ).innerHTML += this.progressBarTemplate(
        this.results.roundResults[i],
        this.results.roundResults[i].Color
      );
    }
  }

  refreshSections(area) {
    let selectSections = document.getElementById("select-sections");
    selectSections.parentNode.style.display = "block";
    for (let i = selectSections.options.length - 1; i >= 0; i--) {
      selectSections.remove(i);
    }
    for (let i in area.Sections) {
      let el = document.createElement("option");
      el.textContent = area.Sections[i].Name;
      el.value = area.Sections[i].ID;
      selectSections.appendChild(el);
    }
  }
}
